/**
 *******************************************************************************
 * @file    flash.c
 * @brief   flash operation APIs' definition
 * @version V1.001
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

#include "flash.h"

uint8_t Addr_To_BN(uint32_t * addr);

/**
  * @brief  write flash
  * @param  destination address, source address, length of data.
  * @retval ERROR or SUCCESS.
  */
uint8_t Write_Flash(uint32_t * addr_dest, uint32_t * addr_source, uint32_t len)
{
    uint32_t size;
    uint32_t *source;
    uint32_t *dest;

    dest = addr_dest;
    source = addr_source;
    size = len;
    while (size > 512U) {
        if (ERR_NONE == FLASH_WritePage(dest, source, 512U)) {  /* write one page every time */
            /* Do nothing */
        } else {
            return ERROR;
        }
        size = size - 512U;
        dest = dest + 512U / 4U;
        source = source + 512U / 4U;
    }
    if (ERR_NONE == FLASH_WritePage(dest, source, (uint16_t) size)) {   /* write the last data, no more than one page */
        /* Do nothing */
    } else {
        return ERROR;
    }
    return SUCCESS;
}

/**
  * @brief  Erase the block
  * @param  block address.
  * @retval success or error code.
  */
uint8_t FLASH_EraseBlock(uint32_t * block_addr)
{
    uint32_t *addr1;
    uint32_t *addr2;
    uint32_t *blk_adr;
    uint32_t counter;

    blk_adr = (uint32_t *) ((uint32_t) block_addr & 0xffff8000U);
    if ((uint32_t) blk_adr > (FLASH_END)) {     /* check if this address is in FLASH space */
        return ERR_FA;
    } else {
        /* do nothing */
    }
#ifdef _BOOT_ROM
    if ((uint32_t) blk_adr < (FLASH_BEGIN)) {   /* check if this address is in FLASH space */
        return ERR_FA;
    } else {
        /* do nothing */
    }
#endif
    if (FC_GetBlockProtectState((FC_BlockNum) Addr_To_BN(blk_adr)) == ENABLE) { /* check if this address is protected */
        return ERR_PRO;
    } else {
        /* do nothing */
    }
    addr1 = (uint32_t *) (FLASH_BEGIN + FLASH_CMD_BC1_ADDR);
    addr2 = (uint32_t *) (FLASH_BEGIN + FLASH_CMD_BC2_ADDR);

    __disable_irq();            /* Disable INT */
    *addr1 = FLASH_CMD_BC1_DATA;        /* bus cycle 1 */
    *addr2 = FLASH_CMD_BC2_DATA;        /* bus cycle 2 */
    *addr1 = 0x00000080U;       /* bus cycle 3 */
    *addr1 = FLASH_CMD_BC1_DATA;        /* bus cycle 4 */
    *addr2 = FLASH_CMD_BC2_DATA;        /* bus cycle 5 */
    *blk_adr = 0x00000030U;     /* bus cycle 6 */
    counter = OVER_TIME;
    while (BUSY == FC_GetBusyState()) { /* check if FLASH is busy with overtime counter */
        if (!(counter--)) {     /* check overtime */
            *addr2 = 0xf0U;     /* Reset FLASH */
            __enable_irq();     /* Enable INT */
            return ERR_OT;
        } else {
            /* do nothing */
        }
    }
    *addr2 = 0xf0U;             /* Reset FLASH */
    __enable_irq();             /* Enable INT */

    return ERR_NONE;
}

/**
  * @brief  Program data no more than 128*4 byte into one page
  * @param  page address, data buffer, size(for bytes).
  * @retval success or error code.
  */
uint8_t FLASH_WritePage(uint32_t * page_addr, uint32_t * data, uint16_t size)
{
    uint32_t *addr1;
    uint32_t *addr2;
    uint32_t *page_adr;
    uint32_t *source;
    uint32_t counter;
    uint16_t i;
    uint16_t len;
    uint32_t mask;
    uint32_t buf;

    len = size;
    if (len > 512U) {           /* check size, no more than one page */
        return ERR_SIZE;
    } else {
        /* do nothing */
    }
    page_adr = (uint32_t *) ((uint32_t) page_addr & 0xfffffe00U);
    if ((uint32_t) page_adr > (FLASH_END)) {    /* check if this address is in FLASH space */
        return ERR_FA;
    } else {
        /* do nothing */
    }
#ifdef _BOOT_ROM
    if ((uint32_t) page_adr < (FLASH_BEGIN)) {  /* check if this address is in FLASH space */
        return ERR_FA;
    } else {
        /* do nothing */
    }
#endif
    if (FC_GetBlockProtectState((FC_BlockNum) Addr_To_BN(page_addr)) == ENABLE) {       /* check if this address is protected */
        return ERR_PRO;
    } else {
        /* do nothing */
    }

    addr1 = (uint32_t *) (FLASH_BEGIN + FLASH_CMD_BC1_ADDR);
    addr2 = (uint32_t *) (FLASH_BEGIN + FLASH_CMD_BC2_ADDR);

    __disable_irq();            /* Disable INT */
    *addr1 = FLASH_CMD_BC1_DATA;        /* bus cycle 1 */
    *addr2 = FLASH_CMD_BC2_DATA;        /* bus cycle 2 */
    *addr1 = 0x000000a0U;       /* bus cycle 3 */
    source = data;
    len = len & 0xfffcU;
    for (i = 0U; i < (len / 4U); i++) { /* bus cycle 4~n */
        *page_adr = *source;    /* 32 bits write */
        source++;
    }
    mask = 0xff000000U;
    buf = *source;
    for (i = 0U; i < (4U + len - size); i++) {
        buf = buf | mask;
        mask = mask >> 8U;
    }
    *page_adr = buf;            /* write the last data(maybe 1,2 or 3 bytes), but not change the other part of the 32 bits */
    for (i = (512U - size) >> 2U; i > 0U; i--) {
        *page_adr = 0xffffffffU;        /* fill the other part of this page with 0xff */
    }
    counter = OVER_TIME;
    while (BUSY == FC_GetBusyState()) { /* check if FLASH is busy with overtime counter */
        if (!(counter--)) {     /* check overtime */
            *addr2 = 0xf0U;     /* Reset FLASH */
            __enable_irq();     /* Enable INT */
            return ERR_OT;
        } else {
            /* do nothing */
        }
    }
    *addr2 = 0xf0U;             /* Reset FLASH */
    __enable_irq();             /* Enable INT */

    return ERR_NONE;
}

/**
  * @brief  Transfer Flash adrress to block number
  * @param  Flash adrress.
  * @retval block number.
  */
uint8_t Addr_To_BN(uint32_t * addr)
{
    uint8_t block;
    uint32_t f_adr;

    f_adr = (uint32_t) addr & (FLASH_END - FLASH_BEGIN);        /* make sure the address is in 0x0000_0000~0x0007_ffff */

    if (f_adr >= 0x60000U) {    /* check which block is this page in */
        /* block 0 */
        block = 0U;
    } else {                    /* block 1~5 */
        f_adr = f_adr >> 15U;
        block = 5U;
        while (f_adr) {
            f_adr = f_adr >> 1U;
            block--;
        }
    }

    return block;
}

/*************************** END OF FILE **************************************/
