/**
 *******************************************************************************
 * @file    main.c
 * @brief   main function of Flash demo(user boot mode) for TMPM330FxFG
 * @version V1.001
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

#include "main.h"

void (*startup) (void);
void delay(uint32_t time);
void LCD_Display(char *First_Line, char *Second_Line);

#if defined ( __CC_ARM   )      /* RealView Compiler */
/**
  * @brief  Get store base of "FLASHSWAP" section
  * @param  None.
  * @retval The base.
  */
__ASM unsigned int __Get_FLASHSWAP_Store_Base(void)
{ 
    IMPORT ||Load$$FLASHSWAP_RAM$$Base|| 
    LDR R0, =||Load$$FLASHSWAP_RAM$$Base|| 
    BX LR
}

/**
  * @brief  Get run base of "FLASHSWAP" section
  * @param  None.
  * @retval The base.
  */
__ASM unsigned int __Get_FLASHSWAP_Run_Base(void)
{
    IMPORT ||Image$$FLASHSWAP_RAM$$RO$$Base|| 
    LDR R0, =||Image$$FLASHSWAP_RAM$$RO$$Base|| 
    BX LR
}

/**
  * @brief  Get run limit of "FLASHSWAP" section
  * @param  None.
  * @retval The limit.
  */
__ASM unsigned int __Get_FLASHSWAP_Run_Limit(void)
{
    IMPORT ||Image$$FLASHSWAP_RAM$$RO$$Limit||
    LDR R0, =||Image$$FLASHSWAP_RAM$$RO$$Limit|| 
    BX LR
}

/**
  * @brief  Reset SP
  * @param  None.
  * @retval None.
  */
__ASM void ResetSP(void)
{
    MOV R0,#0
    LDR SP,[R0]
}
#endif

/**
  * @brief  delay some time
  * @param  delay time
  * @retval None.
  */
void delay(uint32_t time)
{
    uint32_t i;
    for (i = 0U; i < time; i++) {
        /* Do nothing */
    }
}

/**
  * @brief  Display text string on LCD screen
  * @param  text strings of first and second line
  * @retval None.
  */
void LCD_Display(char *First_Line, char *Second_Line)
{
    Send_To_LCD((unsigned char) FIRST_LINE, (unsigned char) LCD_COMMAND);       /* LCD display on first line */
    Send_LCD_Text(First_Line);
    Send_To_LCD((unsigned char) SECOND_LINE, (unsigned char) LCD_COMMAND);      /* LCD display on econd line */
    Send_LCD_Text(Second_Line);
    delay(0x5FFFFU);            /* delay some time */
}

/**
  * @brief  main function to run flash userboot mode demo
  * @param  None.
  * @retval result to retun to system.
  */
int main(void)
{
    CG_SetPLL(DISABLE);         /* Disable PLL */
    CG_SetFcSrc(CG_FC_SRC_FOSC);        /* Select fosc */

    KEY_Configuration();        /* init KEY */
    LCD_Configuration();        /* init LCD */
    LCD_Light(ON);              /* turn on the light of LCD */

    if (Mode_Judgement() == USER_BOOT_MODE) {   /* if KEY1 pressed, enter user boot mode */

        LCD_Display("User Boot Mode", "");
        LCD_Display("RAM transferring", "......");

        Copy_Routine(FLASH_API_RAM, FLASH_API_ROM, SIZE_FLASH_API);     /* copy flash API to RAM */

        LCD_Display("Flash burning", "......");

        if (FC_SUCCESS == Flash_Swap()) {       /* swap the flash content */
            /* Do nothing */
        } else {
            LCD_Display("Error", "");   /* if have any error, display error information */
        }

    } else {                    /* if KEY1 not being pressed, enter common mode */

#if defined ( __CC_ARM   )      /* RealView Compiler */
        ResetSP();              /* reset SP */
#elif defined ( __ICCARM__ )    /* IAR Compiler */
        asm("MOV R0, #0");      /* reset SP */
        asm("LDR SP, [r0]");
#endif

        startup = CODE_START;
        startup();              /* jump to code start address to run */
    }
    return 0;
}

/**
  * @brief  judge current mode is USER_BOOT_MODE or NORMAL_MODE
  * @param  None.
  * @retval Current mode: USER_BOOT_MODE or NORMAL_MODE.
  */
uint8_t Mode_Judgement(void)
{
    return (KEY_Get(KEY1)) ? USER_BOOT_MODE : NORMAL_MODE;
}

/**
  * @brief  copy 32-bit data from source to dest
  * @param  the address of source and dast, the data size
  * @retval None.
  */
void Copy_Routine(uint32_t * dest, uint32_t * source, uint32_t size)
{
    uint32_t *dest_addr, *source_addr, tmpsize;
    uint32_t i, tmps, tmpd, mask;

    dest_addr = dest;
    source_addr = source;

    tmpsize = size >> 2U;

    for (i = 0U; i < tmpsize; i++) {    /* 32bits copy */
        *dest_addr = *source_addr;
        dest_addr++;
        source_addr++;
    }
    if (size & 0x00000003U) {   /* if the last data size is not 0(maybe 1,2 or 3), copy the last data */
        mask = 0xFFFFFF00U;
        i = size & 0x00000003U;
        tmps = *source_addr;
        tmpd = *dest_addr;
        while (i - 1U) {
            mask = mask << 8U;
            i--;
        }
        tmps = tmps & (~mask);
        tmpd = tmpd & (mask);
        *dest_addr = tmps + tmpd;       /* 32bits copy, but only change the bytes need to be changed */
    } else {
        /* Do nothing */
    }
}

/**
  * @brief  swap two code in flash, and runs at RAM, then software reset
  * @param  None.
  * @retval ERROR or SUCCESS.
  */
#if defined ( __CC_ARM   )      /* RealView Compiler */
#pragma arm section code="FLASH_ROM"
#elif defined ( __ICCARM__ )    /* IAR Compiler */
#pragma location = "FLASH_ROM"
#endif
uint8_t Flash_Swap(void)
{
    uint32_t reg_value = 0U;

    Copy_Routine(DEMO_A_RAM, DEMO_A_FLASH, SIZE_DEMO_A);        /* copy A to RAM */
    Copy_Routine(DEMO_B_RAM, DEMO_B_FLASH, SIZE_DEMO_B);        /* copy B to RAM */

    if (FC_SUCCESS == FLASH_EraseBlock(DEMO_A_FLASH)) { /* erase A */
        /* Do nothing */
    } else {
        return ERROR;
    }

    if (FC_SUCCESS == Write_Flash(DEMO_A_FLASH, DEMO_B_RAM, SIZE_DEMO_B)) {     /* write B to A */
        /* Do nothing */
    } else {
        return ERROR;
    }

    if (FC_SUCCESS == FLASH_EraseBlock(DEMO_B_FLASH)) { /* erase B */
        /* Do nothing */
    } else {
        return ERROR;
    }

    if (FC_SUCCESS == Write_Flash(DEMO_B_FLASH, DEMO_A_RAM, SIZE_DEMO_A)) {     /* write A to B */
        /* Do nothing */
    } else {
        return ERROR;
    }

    LCD_Display("Finished", "Restart......");

    while (KEY_Get(KEY1)) {     /* wait for KEY1 release */
    }

    reg_value = SCB->AIRCR;     /* software reset */
    reg_value = (uint32_t) 0x05FA0001;
    SCB->AIRCR = reg_value;

    return SUCCESS;
}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif

/*************************** END OF FILE **************************************/
